@@ -26,14 +26,17 @@
 
 #include <stdint.h>
 
+#include "libavutil/qsort.h"
+#include"libavutil/common.h"
+
 #include "avcodec.h"
-#include "get_bits.h"
 #include "huffman.h"
+#include "vlc.h"
 
 /* symbol for Huffman tree node */
 #define HNODE -1
 
-typedef struct {
+typedef struct HeapElem {
     uint64_t val;
     int name;
 } HeapElem;
@@ -52,23 +55,31 @@ static void heap_sift(HeapElem *h, int root, int size)
     }
 }
 
-int ff_huff_gen_len_table(uint8_t *dst, const uint64_t *stats, int size)
+int ff_huff_gen_len_table(uint8_t *dst, const uint64_t *stats, int stats_size, int skip0)
 {
-    HeapElem *h  = av_malloc(sizeof(*h) * size);
-    int *up      = av_malloc(sizeof(*up) * 2 * size);
-    uint8_t *len = av_malloc(sizeof(*len) * 2 * size);
+    HeapElem *h  = av_malloc_array(sizeof(*h), stats_size);
+    int *up      = av_malloc_array(sizeof(*up) * 2, stats_size);
+    uint8_t *len = av_malloc_array(sizeof(*len) * 2, stats_size);
+    uint16_t *map= av_malloc_array(sizeof(*map), stats_size);
     int offset, i, next;
+    int size = 0;
     int ret = 0;
 
-    if (!h || !up || !len) {
+    if (!h || !up || !len || !map) {
         ret = AVERROR(ENOMEM);
         goto end;
     }
 
+    for (i = 0; i<stats_size; i++) {
+        dst[i] = 255;
+        if (stats[i] || !skip0)
+            map[size++] = i;
+    }
+
     for (offset = 1; ; offset <<= 1) {
         for (i=0; i < size; i++) {
             h[i].name = i;
-            h[i].val = (stats[i] << 14) + offset;
+            h[i].val = (stats[map[i]] << 14) + offset;
         }
         for (i = size / 2 - 1; i >= 0; i--)
             heap_sift(h, i, size);
@@ -89,8 +100,8 @@ int ff_huff_gen_len_table(uint8_t *dst, const uint64_t *stats, int size)
         for (i = 2 * size - 3; i >= size; i--)
             len[i] = len[up[i]] + 1;
         for (i = 0; i < size; i++) {
-            dst[i] = len[up[i]] + 1;
-            if (dst[i] >= 32) break;
+            dst[map[i]] = len[up[i]] + 1;
+            if (dst[map[i]] >= 32) break;
         }
         if (i==size) break;
     }
@@ -98,6 +109,7 @@ end:
     av_free(h);
     av_free(up);
     av_free(len);
+    av_free(map);
     return ret;
 }
 
@@ -161,7 +173,7 @@ int ff_huff_build_tree(AVCodecContext *avctx, VLC *vlc, int nb_codes, int nb_bit
                "Tree construction is not possible\n");
         return -1;
     }
-    qsort(nodes, nb_codes, sizeof(Node), cmp);
+    AV_QSORT(nodes, nb_codes, Node, cmp);
     cur_node = nb_codes;
     nodes[nb_codes*2-1].count = 0;
     for (i = 0; i < nb_codes * 2 - 1; i += 2) {
