@@ -30,6 +30,7 @@
 
 #define BITSTREAM_READER_LE
 #include "libavutil/attributes.h"
+#include "libavutil/imgutils.h"
 #include "libavutil/timer.h"
 #include "avcodec.h"
 #include "get_bits.h"
@@ -310,7 +311,7 @@ av_cold int ff_ivi_init_planes(IVIPlaneDesc *planes, const IVIPicConfig *cfg,
 
     ivi_free_buffers(planes);
 
-    if (cfg->pic_width < 1 || cfg->pic_height < 1 ||
+    if (av_image_check_size(cfg->pic_width, cfg->pic_height, 0, NULL) < 0 ||
         cfg->luma_bands < 1 || cfg->chroma_bands < 1)
         return AVERROR_INVALIDDATA;
 
@@ -1083,7 +1084,7 @@ int ff_ivi_decode_frame(AVCodecContext *avctx, void *data, int *got_frame,
     }
 
     if (ctx->gop_flags & IVI5_IS_PROTECTED) {
-        avpriv_report_missing_feature(avctx, "Password-protected clip!\n");
+        avpriv_report_missing_feature(avctx, "Password-protected clip");
         return AVERROR_PATCHWELCOME;
     }
 
@@ -1171,6 +1172,22 @@ int ff_ivi_decode_frame(AVCodecContext *avctx, void *data, int *got_frame,
         }
     }
 
+    if (ctx->show_indeo4_info) {
+        if (ctx->is_scalable)
+            av_log(avctx, AV_LOG_DEBUG, "This video uses scalability mode\n");
+        if (ctx->uses_tiling)
+            av_log(avctx, AV_LOG_DEBUG, "This video uses local decoding\n");
+        if (ctx->has_b_frames)
+            av_log(avctx, AV_LOG_DEBUG, "This video contains B-frames\n");
+        if (ctx->has_transp)
+            av_log(avctx, AV_LOG_DEBUG, "Transparency mode is enabled\n");
+        if (ctx->uses_haar)
+            av_log(avctx, AV_LOG_DEBUG, "This video uses Haar transform\n");
+        if (ctx->uses_fullpel)
+            av_log(avctx, AV_LOG_DEBUG, "This video uses fullpel motion vectors\n");
+        ctx->show_indeo4_info = 0;
+    }
+
     return buf_size;
 }
 
@@ -1186,23 +1203,6 @@ av_cold int ff_ivi_decode_close(AVCodecContext *avctx)
     if (ctx->mb_vlc.cust_tab.table)
         ff_free_vlc(&ctx->mb_vlc.cust_tab);
 
-#if IVI4_STREAM_ANALYSER
-    if (ctx->is_indeo4) {
-    if (ctx->is_scalable)
-        av_log(avctx, AV_LOG_ERROR, "This video uses scalability mode!\n");
-    if (ctx->uses_tiling)
-        av_log(avctx, AV_LOG_ERROR, "This video uses local decoding!\n");
-    if (ctx->has_b_frames)
-        av_log(avctx, AV_LOG_ERROR, "This video contains B-frames!\n");
-    if (ctx->has_transp)
-        av_log(avctx, AV_LOG_ERROR, "Transparency mode is enabled!\n");
-    if (ctx->uses_haar)
-        av_log(avctx, AV_LOG_ERROR, "This video uses Haar transform!\n");
-    if (ctx->uses_fullpel)
-        av_log(avctx, AV_LOG_ERROR, "This video uses fullpel motion vectors!\n");
-    }
-#endif
-
     av_frame_free(&ctx->p_frame);
 
     return 0;
