@@ -170,7 +170,7 @@ static inline void put_bits(PutBitContext *s, int n, unsigned int value)
             av_log(NULL, AV_LOG_ERROR, "Internal error, put_bits buffer too small\n");
             av_assert2(0);
         }
-        bit_buf     = (bit_left == 32) ? 0 : value >> bit_left;
+        bit_buf     = value >> bit_left;
         bit_left   += 32;
     }
     bit_left -= n;
@@ -201,7 +201,7 @@ static inline void put_sbits(PutBitContext *pb, int n, int32_t value)
 {
     av_assert2(n >= 0 && n <= 31);
 
-    put_bits(pb, n, value & ((1 << n) - 1));
+    put_bits(pb, n, av_mod_uintp2(value, n));
 }
 
 /**
@@ -260,6 +260,7 @@ static inline void skip_put_bits(PutBitContext *s, int n)
  */
 static inline void set_put_bits_buffer_size(PutBitContext *s, int size)
 {
+    av_assert0(size <= INT_MAX/8 - 32);
     s->buf_end = s->buf + size;
     s->size_in_bits = 8*size;
 }
