@@ -26,19 +26,19 @@
 #include "internal.h"
 
 static const AVCodecTag rsd_tags[] = {
-    { AV_CODEC_ID_ADPCM_THP,       MKTAG('G','A','D','P') },
+    { AV_CODEC_ID_ADPCM_PSX,       MKTAG('V','A','G',' ') },
+    { AV_CODEC_ID_ADPCM_THP_LE,    MKTAG('G','A','D','P') },
+    { AV_CODEC_ID_ADPCM_THP,       MKTAG('W','A','D','P') },
     { AV_CODEC_ID_ADPCM_IMA_RAD,   MKTAG('R','A','D','P') },
+    { AV_CODEC_ID_ADPCM_IMA_WAV,   MKTAG('X','A','D','P') },
     { AV_CODEC_ID_PCM_S16BE,       MKTAG('P','C','M','B') },
     { AV_CODEC_ID_PCM_S16LE,       MKTAG('P','C','M',' ') },
+    { AV_CODEC_ID_XMA2,            MKTAG('X','M','A',' ') },
     { AV_CODEC_ID_NONE, 0 },
 };
 
 static const uint32_t rsd_unsupported_tags[] = {
     MKTAG('O','G','G',' '),
-    MKTAG('V','A','G',' '),
-    MKTAG('W','A','D','P'),
-    MKTAG('X','A','D','P'),
-    MKTAG('X','M','A',' '),
 };
 
 static int rsd_probe(AVProbeData *p)
@@ -55,8 +55,8 @@ static int rsd_probe(AVProbeData *p)
 static int rsd_read_header(AVFormatContext *s)
 {
     AVIOContext *pb = s->pb;
-    int i, version, start = 0x800;
-    AVCodecContext *codec;
+    int i, ret, version, start = 0x800;
+    AVCodecParameters *par;
     AVStream *st = avformat_new_stream(s, NULL);
 
     if (!st)
@@ -65,16 +65,16 @@ static int rsd_read_header(AVFormatContext *s)
     avio_skip(pb, 3); // "RSD"
     version = avio_r8(pb) - '0';
 
-    codec = st->codec;
-    codec->codec_type = AVMEDIA_TYPE_AUDIO;
-    codec->codec_tag  = avio_rl32(pb);
-    codec->codec_id   = ff_codec_get_id(rsd_tags, codec->codec_tag);
-    if (!codec->codec_id) {
-        char tag_buf[5];
+    par = st->codecpar;
+    par->codec_type = AVMEDIA_TYPE_AUDIO;
+    par->codec_tag  = avio_rl32(pb);
+    par->codec_id   = ff_codec_get_id(rsd_tags, par->codec_tag);
+    if (!par->codec_id) {
+        char tag_buf[32];
 
-        av_get_codec_tag_string(tag_buf, sizeof(tag_buf), codec->codec_tag);
+        av_get_codec_tag_string(tag_buf, sizeof(tag_buf), par->codec_tag);
         for (i=0; i < FF_ARRAY_ELEMS(rsd_unsupported_tags); i++) {
-            if (codec->codec_tag == rsd_unsupported_tags[i]) {
+            if (par->codec_tag == rsd_unsupported_tags[i]) {
                 avpriv_request_sample(s, "Codec tag: %s", tag_buf);
                 return AVERROR_PATCHWELCOME;
             }
@@ -83,37 +83,68 @@ static int rsd_read_header(AVFormatContext *s)
         return AVERROR_INVALIDDATA;
     }
 
-    codec->channels = avio_rl32(pb);
-    if (!codec->channels)
+    par->channels = avio_rl32(pb);
+    if (!par->channels)
         return AVERROR_INVALIDDATA;
 
     avio_skip(pb, 4); // Bit depth
-    codec->sample_rate = avio_rl32(pb);
-    if (!codec->sample_rate)
+    par->sample_rate = avio_rl32(pb);
+    if (!par->sample_rate)
         return AVERROR_INVALIDDATA;
 
     avio_skip(pb, 4); // Unknown
 
-    switch (codec->codec_id) {
-    case AV_CODEC_ID_ADPCM_IMA_RAD:
-        codec->block_align = 20 * codec->channels;
-        if (pb->seekable)
-            st->duration = av_get_audio_frame_duration(codec, avio_size(pb) - start);
+    switch (par->codec_id) {
+    case AV_CODEC_ID_XMA2:
+        par->block_align = 2048;
+        ff_alloc_extradata(par, 34);
+        if (!par->extradata)
+            return AVERROR(ENOMEM);
+        memset(par->extradata, 0, 34);
         break;
-    case AV_CODEC_ID_ADPCM_THP:
+    case AV_CODEC_ID_ADPCM_PSX:
+        par->block_align = 16 * par->channels;
+        if (pb->seekable)
+            st->duration = av_get_audio_frame_duration2(par, avio_size(pb) - start);
+        break;
+    case AV_CODEC_ID_ADPCM_IMA_RAD:
+        par->block_align = 20 * par->channels;
+        if (pb->seekable)
+            st->duration = av_get_audio_frame_duration2(par, avio_size(pb) - start);
+        break;
+    case AV_CODEC_ID_ADPCM_IMA_WAV:
+        if (version == 2)
+            start = avio_rl32(pb);
+
+        par->bits_per_coded_sample = 4;
+        par->block_align = 36 * par->channels;
+        if (pb->seekable)
+            st->duration = av_get_audio_frame_duration2(par, avio_size(pb) - start);
+        break;
+    case AV_CODEC_ID_ADPCM_THP_LE:
         /* RSD3GADP is mono, so only alloc enough memory
            to store the coeff table for a single channel. */
 
         start = avio_rl32(pb);
 
-        if (ff_get_extradata(codec, s->pb, 32) < 0)
-            return AVERROR(ENOMEM);
-
-        for (i = 0; i < 16; i++)
-            AV_WB16(codec->extradata + i * 2, AV_RL16(codec->extradata + i * 2));
-
+        if ((ret = ff_get_extradata(s, par, s->pb, 32)) < 0)
+            return ret;
         if (pb->seekable)
-            st->duration = (avio_size(pb) - start) / 8 * 14;
+            st->duration = av_get_audio_frame_duration2(par, avio_size(pb) - start);
+        break;
+    case AV_CODEC_ID_ADPCM_THP:
+        par->block_align = 8 * par->channels;
+        avio_skip(s->pb, 0x1A4 - avio_tell(s->pb));
+
+        if ((ret = ff_alloc_extradata(st->codecpar, 32 * par->channels)) < 0)
+            return ret;
+
+        for (i = 0; i < par->channels; i++) {
+            avio_read(s->pb, st->codecpar->extradata + 32 * i, 32);
+            avio_skip(s->pb, 8);
+        }
+        if (pb->seekable)
+            st->duration = (avio_size(pb) - start) / (8 * par->channels) * 14;
         break;
     case AV_CODEC_ID_PCM_S16LE:
     case AV_CODEC_ID_PCM_S16BE:
@@ -121,37 +152,58 @@ static int rsd_read_header(AVFormatContext *s)
             start = avio_rl32(pb);
 
         if (pb->seekable)
-            st->duration = (avio_size(pb) - start) / 2 / codec->channels;
+            st->duration = (avio_size(pb) - start) / 2 / par->channels;
         break;
     }
 
     avio_skip(pb, start - avio_tell(pb));
+    if (par->codec_id == AV_CODEC_ID_XMA2) {
+        avio_skip(pb, avio_rb32(pb) + avio_rb32(pb));
+        st->duration = avio_rb32(pb);
+    }
 
-    avpriv_set_pts_info(st, 64, 1, codec->sample_rate);
+    avpriv_set_pts_info(st, 64, 1, par->sample_rate);
 
     return 0;
 }
 
 static int rsd_read_packet(AVFormatContext *s, AVPacket *pkt)
 {
-    AVCodecContext *codec = s->streams[0]->codec;
+    AVCodecParameters *par = s->streams[0]->codecpar;
     int ret, size = 1024;
+    int64_t pos;
 
     if (avio_feof(s->pb))
         return AVERROR_EOF;
 
-    if (codec->codec_id == AV_CODEC_ID_ADPCM_IMA_RAD)
-        ret = av_get_packet(s->pb, pkt, codec->block_align);
-    else
-        ret = av_get_packet(s->pb, pkt, size);
+    pos = avio_tell(s->pb);
+    if (par->codec_id == AV_CODEC_ID_ADPCM_IMA_RAD ||
+        par->codec_id == AV_CODEC_ID_ADPCM_PSX     ||
+        par->codec_id == AV_CODEC_ID_ADPCM_IMA_WAV ||
+        par->codec_id == AV_CODEC_ID_XMA2) {
+        ret = av_get_packet(s->pb, pkt, par->block_align);
+    } else if (par->codec_tag == MKTAG('W','A','D','P') &&
+               par->channels > 1) {
+        int i, ch;
 
-    if (ret != size) {
-        if (ret < 0) {
-            av_free_packet(pkt);
+        ret = av_new_packet(pkt, par->block_align);
+        if (ret < 0)
             return ret;
+        for (i = 0; i < 4; i++) {
+            for (ch = 0; ch < par->channels; ch++) {
+                pkt->data[ch * 8 + i * 2 + 0] = avio_r8(s->pb);
+                pkt->data[ch * 8 + i * 2 + 1] = avio_r8(s->pb);
+            }
         }
-        av_shrink_packet(pkt, ret);
+        ret = 0;
+    } else {
+        ret = av_get_packet(s->pb, pkt, size);
     }
+
+    if (par->codec_id == AV_CODEC_ID_XMA2 && pkt->size >= 1)
+        pkt->duration = (pkt->data[0] >> 2) * 512;
+
+    pkt->pos = pos;
     pkt->stream_index = 0;
 
     return ret;
@@ -165,4 +217,5 @@ AVInputFormat ff_rsd_demuxer = {
     .read_packet    =   rsd_read_packet,
     .extensions     =   "rsd",
     .codec_tag      =   (const AVCodecTag* const []){rsd_tags, 0},
+    .flags          =   AVFMT_GENERIC_INDEX,
 };
